@@ -4,12 +4,13 @@ use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 use Cassandra::Client::Config;
 use Cassandra::Client::Connection;
 use Cassandra::Client::Util qw/series/;
 use Cassandra::Client::AsyncIO;
+use Cassandra::Client::AsyncAnyEvent;
 use Cassandra::Client::Metadata;
 use Cassandra::Client::Pool;
 use List::Util qw/shuffle/;
@@ -29,7 +30,8 @@ sub new {
     my $options= Cassandra::Client::Config->new(
         \%args
     );
-    my $async_io= Cassandra::Client::AsyncIO->new(
+    my $async_class= $options->{anyevent} ? "Cassandra::Client::AsyncAnyEvent" : "Cassandra::Client::AsyncIO";
+    my $async_io= $async_class->new(
         options => $options,
     );
     my $metadata= Cassandra::Client::Metadata->new(
@@ -169,72 +171,154 @@ sub _handle_status_change {
 sub _prepare {
     my ($self, $callback, $query)= @_;
 
-    return _cb($callback, "Cannot prepare: not connected") if !$self->{connected};
-
-    # XXX Do we need to retry prepares on other hosts if they fail for reasons other than bad CQL?
-
+    # Fast path: we're already done
     if ($self->{metadata}->is_prepared(\$query)) {
         return _cb($callback);
     }
 
-    my $connection= $self->{pool}->get_one;
-    if (!$connection) {
-        $self->{pool}->get_one_cb(sub {
-            my ($error, $connection)= @_;
-            return _cb($callback, $error) if $error;
+    # XXX: Do we really need the duplication of having a second fast path for prepare()?
+    goto SLOWPATH if !$self->{connected};
 
-            # Copy/paste from below
-            $connection->prepare(sub {
-                my ($error)= @_;
-                return _cb($callback, $error);
-            }, $query);
-        });
-        return;
-    }
+    my $connection= $self->{pool}->get_one;
+    goto SLOWPATH if !$connection;
 
     $connection->prepare(sub {
         my ($error)= @_;
-        return _cb($callback, $error);
+        if ($error && ref($error) && $error->retryable) {
+            return $self->_prepare_slowpath($callback, $query);
+        } else {
+            return _cb($callback, $error);
+        }
     }, $query);
 
     return;
+
+SLOWPATH:
+    return $self->_prepare_slowpath($callback, $query);
+}
+
+sub _prepare_slowpath {
+    my ($self, $callback, $query)= @_;
+
+    # XXX Do we need to retry prepares on other hosts if they fail for reasons other than bad CQL?
+
+    series([
+        sub {
+            my ($next)= @_;
+            $self->_connect($next);
+        }, sub {
+            my ($next)= @_;
+            $self->{pool}->get_one_cb($next);
+        }, sub {
+            my ($next, $connection)= @_;
+            $connection->prepare($next);
+        }
+    ], sub {
+        my ($error)= @_;
+        return _cb($callback, $error);
+    });
+    return;
 }
 
 sub _execute {
     my ($self, $callback, $query, $params, $attribs)= @_;
 
-    return _cb($callback, "Cannot execute: not connected") if !$self->{connected};
-
     my $params_copy= $params ? clone($params) : undef;
     my $attribs_copy= $attribs ? clone($attribs) : undef;
 
-    # XXX We don't retry yet.
+    goto SLOWPATH if !$self->{connected};
 
     my $connection= $self->{pool}->get_one;
-    if (!$connection) {
-        $self->{pool}->get_one_cb(sub {
-            my ($error, $connection)= @_;
-            return _cb($callback, $error) if $error;
-
-            # Copy/paste from below
-            $connection->execute_prepared(sub {
-                my ($error, $result)= @_;
-                _cb($callback, $error, $result);
-                return;
-            }, \$query, $params_copy, $attribs_copy);
-        });
-        return;
-    }
+    goto SLOWPATH if !$connection;
 
     $connection->execute_prepared(sub {
         my ($error, $result)= @_;
-        _cb($callback, $error, $result);
+        if ($error && ref($error) && $error->retryable) {
+            return $self->_execute_slowpath($callback, $query, $params_copy, $attribs_copy);
+        } else {
+            _cb($callback, $error, $result);
+        }
         return;
     }, \$query, $params_copy, $attribs_copy);
 
     return;
+
+SLOWPATH:
+    return $self->_execute_slowpath($callback, $query, $params_copy, $attribs_copy);
 }
 
+sub _execute_slowpath {
+    my ($self, $callback, $query, $params_copy, $attribs_copy)= @_;
+
+    series([
+        sub {
+            my ($next)= @_;
+            $self->_connect($next);
+        }, sub {
+            my ($next)= @_;
+            $self->{pool}->get_one_cb($next);
+        }, sub {
+            my ($next, $connection)= @_;
+            $connection->execute_prepared($next, \$query, $params_copy, $attribs_copy);
+        }
+    ], sub {
+        my ($error, $result)= @_;
+        _cb($callback, $error, $result);
+        return;
+    });
+    return;
+}
+
+sub _batch {
+    my ($self, $callback, $queries, $attribs)= @_;
+
+    my $queries_copy= $queries ? clone($queries) : undef;
+    my $attribs_copy= $attribs ? clone($attribs) : undef;
+
+    goto SLOWPATH if !$self->{connected};
+
+    my $connection= $self->{pool}->get_one;
+    goto SLOWPATH if !$connection;
+
+    $connection->execute_batch(sub {
+        my ($error, $result)= @_;
+        if ($error && ref($error) && $error->retryable) {
+            return $self->_batch_slowpath($callback, $queries_copy, $attribs_copy);
+        } else {
+            _cb($callback, $error, $result);
+        }
+        return;
+    }, $queries_copy, $attribs_copy);
+
+    return;
+
+SLOWPATH:
+    return $self->_batch_slowpath($callback, $queries_copy, $attribs_copy);
+}
+
+sub _batch_slowpath {
+    my ($self, $callback, $queries_copy, $attribs_copy)= @_;
+
+    series([
+        sub {
+            my ($next)= @_;
+            $self->_connect($next);
+        }, sub {
+            my ($next)= @_;
+            $self->{pool}->get_one_cb($next);
+        }, sub {
+            my ($next, $connection)= @_;
+            $connection->execute_batch($next, $queries_copy, $attribs_copy);
+        }
+    ], sub {
+        my ($error, $result)= @_;
+        _cb($callback, $error, $result);
+        return;
+    });
+    return;
+}
+
+# Utility functions that wrap query functions
 sub _each_page {
     my ($self, $callback, $query, $params, $attribs, $page_callback)= @_;
 
@@ -264,40 +348,6 @@ sub _each_page {
     return;
 }
 
-sub _batch {
-    my ($self, $callback, $queries, $attribs)= @_;
-    return _cb($callback, "Cannot batch: not connected") if !$self->{connected};
-
-    my $queries_copy= $queries ? clone($queries) : undef;
-    my $attribs_copy= $attribs ? clone($attribs) : undef;
-
-    # XXX We don't retry yet.
-
-    my $connection= $self->{pool}->get_one;
-    if (!$connection) {
-        $self->{pool}->get_one_cb(sub {
-            my ($error, $connection)= @_;
-            return _cb($callback, $error) if $error;
-
-            # Copy/paste from below
-            $connection->execute_batch(sub {
-                my ($error, $result)= @_;
-                _cb($callback, $error, $result);
-                return;
-            }, $queries_copy, $attribs_copy);
-        });
-        return;
-    }
-
-    $connection->execute_batch(sub {
-        my ($error, $result)= @_;
-        _cb($callback, $error, $result);
-        return;
-    }, $queries_copy, $attribs_copy);
-
-    return;
-}
-
 sub _wait_for_schema_agreement {
     my ($self, $callback)= @_;
 
@@ -620,6 +670,24 @@ Upon errors, the coderef will die, just like the synchronous methods would. Beca
 
 When used properly, coderefs can give a modest performance boost, but their real value is in the ease of use compared to promises.
 
+=head1 CAVEATS, BUGS, TODO
+
+=over
+
+=item *
+
+Thread support is untested. Use at your own risk.
+
+=item *
+
+The C<timestamp> format is implemented naively by returning
+milliseconds since the UNIX epoch. In Perl you get this number through
+C<time() * 1000>. Trying to save times as C<DateTime> objects or
+strings will not work, and will likely result in warnings and
+unexpected behavior.
+
+=back
+
 =head1 LICENSE
 
 Copyright (C) 2016 L<Tom van der Woerdt|mailto:tvdw@cpan.org>
