@@ -1,12 +1,9 @@
 package Cassandra::Client::AsyncAnyEvent;
-our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::AsyncAnyEvent::VERSION = '0.14_003'; # TRIAL
-
-$Cassandra::Client::AsyncAnyEvent::VERSION = '0.14003';use 5.010;
+use 5.008;
 use strict;
 use warnings;
 
-use Time::HiRes qw(CLOCK_MONOTONIC);
+use Time::HiRes qw();
 use vars qw/@TIMEOUTS/;
 
 sub new {
@@ -35,11 +32,7 @@ sub register {
 sub unregister {
     my ($self, $fh)= @_;
     delete $self->{fh_to_obj}{$fh};
-    if ($self->{timeouts} && grep { $_->[1] == $fh && !$_->[3] } @{$self->{timeouts}}) {
-        warn 'In unregister(): not all timeouts were dismissed!';
-    }
-    @{$self->{timeouts}}= grep { $_->[1] != $fh } @{$self->{timeouts}};
-    undef $self->{ae_timeout} unless @{$self->{timeouts}};
+    @{$self->{timeouts}}= grep { $_->[1] != $fh } @{$self->{timeouts}} if $self->{timeouts};
     return;
 }
 
@@ -95,11 +88,11 @@ sub deadline {
         $self->{ae_timeout}= AnyEvent->timer(
             after => $self->{timer_granularity},
             interval => $self->{timer_granularity},
-            cb => sub { $self->handle_timeouts(Time::HiRes::clock_gettime(CLOCK_MONOTONIC)) },
+            cb => sub { $self->handle_timeouts(Time::HiRes::time()) },
         );
     }
 
-    my $curtime= Time::HiRes::clock_gettime(CLOCK_MONOTONIC);
+    my $curtime= Time::HiRes::time;
     my $deadline= $curtime + $timeout;
     my $additem= [ $deadline, $fh, $id, 0 ];
 
@@ -120,14 +113,12 @@ sub handle_timeouts {
 
     local *TIMEOUTS= $self->{timeouts};
 
-    my %triggered_read;
     while (@TIMEOUTS && $curtime >= $TIMEOUTS[0][0]) {
         my $item= shift @TIMEOUTS;
         if (!$item->[3]) { # If it timed out
             my ($deadline, $fh, $id, $timedout)= @$item;
             my $obj= $self->{fh_to_obj}{$fh};
-            $obj->can_read unless $triggered_read{$fh}++;
-            $obj->can_timeout($id) unless $item->[3]; # We may have received an answer...
+            $obj->can_timeout($id);
         }
     }
 
@@ -138,19 +129,6 @@ sub handle_timeouts {
     return;
 }
 
-sub timer {
-    my ($self, $callback, $wait)= @_;
-    my $t; $t= AE::timer($wait, 0, sub {
-        undef $t;
-        $callback->();
-    });
-}
-
-sub later {
-    my ($self, $callback)= @_;
-    &AE::postpone($callback);
-}
-
 # $something->($async->wait(my $w)); my ($error, $result)= $w->();
 sub wait {
     my ($self)= @_;
@@ -172,28 +150,3 @@ sub wait {
 }
 
 1;
-
-__END__
-
-=pod
-
-=head1 NAME
-
-Cassandra::Client::AsyncAnyEvent
-
-=head1 VERSION
-
-version 0.14_003
-
-=head1 AUTHOR
-
-Tom van der Woerdt <tvdw@cpan.org>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2019 by Tom van der Woerdt.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
-=cut
