@@ -660,8 +660,7 @@ sub request {
 
         my $data= pack('CCsCN/a', 3, $flags, $stream_id, $opcode, $_[3]);
 
-        if (length $self->{pending_write}) {
-            # XXX Now that we have timeouts, we should consider not sending anything if we already timed out
+        if (defined $self->{pending_write}) {
             $self->{pending_write} .= $data;
             last WRITE;
         }
@@ -733,7 +732,11 @@ READ_NEXT:
                 my ($code, $message)= unpack('Nn/a', $body);
                 my ($cb, $dl)= @$stream_cb;
                 $$dl= 1;
-                $cb->(Cassandra::Client::Error->new($code, $message));
+                $cb->(Cassandra::Client::Error->new(
+                    code => $code,
+                    message => $message,
+                    our_fault => ( $code != 0x1000 && $code != 0x1001 && $code != 0x1002 && $code != 0x1003 && $code != 0x1100 && $code != 0x1200 ),
+                ));
 
             } else {
                 my ($cb, $dl)= @$stream_cb;
@@ -768,7 +771,6 @@ sub can_write {
         }
 
         my $error= "$!";
-        undef $self->{pending_write};
         return $self->shutdown(undef, $error);
     }
     if ($result == 0) { return; } # No idea whether that happens, but guard anyway.
@@ -776,6 +778,7 @@ sub can_write {
 
     if (!length $self->{pending_write}) {
         $self->{async_io}->unregister_write($self->{fileno});
+        delete $self->{pending_write};
     }
 
     return;
@@ -801,16 +804,22 @@ sub shutdown {
 
     my $pending= $self->{pending_streams};
     $self->{pending_streams}= {};
-    $_->[0]->("Disconnected: $shutdown_reason") for values %$pending;
 
     $self->{socket}->close;
     $self->{async_io}->unregister_read($self->{fileno});
-    if (length(delete $self->{pending_write})) {
+    if (defined(delete $self->{pending_write})) {
         $self->{async_io}->unregister_write($self->{fileno});
     }
     $self->{async_io}->unregister($self->{fileno}, $self);
     $self->{client}->_disconnected($self->get_pool_id);
 
+    for (values %$pending) {
+        $_->[0]->(Cassandra::Client::Error->new(
+            message   => "Disconnected: $shutdown_reason",
+            our_fault => 0,
+        ));
+    }
+
     $cb->() if $cb;
 
     return;
