@@ -3,7 +3,8 @@ use 5.008;
 use strict;
 use warnings;
 
-use Cassandra::Client::Protocol ':constants';
+use Cassandra::Client::Protocol qw/:constants BIGINT_SUPPORTED pack_long/;
+use Math::BigInt;
 
 use Exporter 'import';
 our @EXPORT_OK= qw/
@@ -12,20 +13,22 @@ our @EXPORT_OK= qw/
 
 use vars qw/@INPUT/;
 
+my @bigint_enc= BIGINT_SUPPORTED ? ( 'q>', 8 ) : ( \&e_bigint );
+
 my %types= (
     TYPE_CUSTOM     ,=> [ \&e_passthru ],
     TYPE_ASCII      ,=> [ \&e_passthru ],
-    TYPE_BIGINT     ,=> [ 'q>', 8 ],
+    TYPE_BIGINT     ,=> [ @bigint_enc  ],
     TYPE_BLOB       ,=> [ \&e_passthru ],
     TYPE_BOOLEAN    ,=> [ \&e_bool ],
-    TYPE_COUNTER    ,=> [ 'q>', 8 ],
-    ##TYPE_DECIMAL    ,=>
+    TYPE_COUNTER    ,=> [ @bigint_enc  ],
+    TYPE_DECIMAL    ,=> [ \&e_decimal  ],
     TYPE_DOUBLE     ,=> [ 'd>', 8 ],
     TYPE_FLOAT      ,=> [ 'f>', 4 ],
     TYPE_INT        ,=> [ 'l>', 4 ],
     TYPE_TEXT       ,=> [ \&e_string ],
-    ##TYPE_VARINT     ,=>
-    TYPE_TIMESTAMP  ,=> [ 'q>', 8 ],
+    TYPE_VARINT     ,=> [ \&e_varint   ],
+    TYPE_TIMESTAMP  ,=> [ @bigint_enc  ],
     TYPE_UUID       ,=> [ \&e_uuid ],
     TYPE_VARCHAR    ,=> [ \&e_string ],
     TYPE_TIMEUUID   ,=> [ \&e_uuid ],
@@ -184,4 +187,55 @@ EOC
     return $code;
 }
 
+sub e_bigint {
+    my ($type, $input, $output)= @_;
+    return "$output .= pack('l>', 8).pack_long($input);";
+}
+
+sub e_varint {
+    my ($type, $input, $output)= @_;
+    return <<EOC;
+{
+    my \$number= Math::BigInt->new($input);
+    my \$negative= \$number->is_neg;
+    if (\$negative) {
+        \$number *= -1; # This means invert the bits and add one
+        \$number -= 1; # So remove that one
+    }
+    my \$hex= \$number->as_hex;
+    \$hex =~ s/^0x//;
+    \$hex= "0\$hex" if length(\$hex) % 2;
+    \$hex= "00\$hex" if substr(\$hex, 0, 1) =~ /[89abcdef]/;
+    my \$binary= pack('H*', \$hex);
+    \$binary= ~\$binary if \$negative;
+    $output .= pack('l>/a', \$binary);
+}
+EOC
+}
+
+sub e_decimal {
+    my ($type, $input, $output)= @_;
+    my $varint_enc= e_varint($type, '$unscaled', '$output_with_length_bytes');
+    return <<EOC;
+my \$input_copy= "".$input;
+my \$scale= 0;
+if (\$input_copy =~ /E([+-]?\\d+)\$/i) {
+    \$scale -= \$1;
+    \$input_copy =~ s/E([+-]?\\d+)\$//i;
+}
+if (\$input_copy =~ /[.](\\d+)\$/) {
+    \$scale += length \$1;
+    \$input_copy =~ s/[.]//;
+}
+\$input_copy =~ s/^0+//;
+\$input_copy= "0" unless length \$input_copy;
+my \$unscaled= \$input_copy;
+my \$output_with_length_bytes;
+VARINT: {
+$varint_enc
+}
+$output .= pack('l>/a', pack('l>', \$scale). substr(\$output_with_length_bytes, 4));
+EOC
+}
+
 1;
